@@ -1,7 +1,7 @@
 """
  Assertions for testify.
 """
-from testify.assertions import assert_not_reached
+from testify.assertions import assert_not_reached, assert_in
 
 
 def assert_raises(expected_exception_class, callable_obj, *args, **kwargs):
@@ -26,4 +26,10 @@ def assert_call(turtle, call_idx, *args, **kwargs):
     """Assert that a function was called on turtle with the correct args."""
     actual = turtle.calls[call_idx] if turtle.calls else None
     msg = "Call %s expected %s, was %s" % (call_idx, (args, kwargs), actual)
-    assert actual == (args, kwargs), msg
\ No newline at end of file
+    assert actual == (args, kwargs), msg
+
+
+def assert_mock_calls(expected, mock_calls):
+    """Assert that all expected calls are in the list of mock_calls."""
+    for expected_call in expected:
+        assert_in(expected_call, mock_calls)
\ No newline at end of file
