@@ -1,20 +1,23 @@
-from testify import TestCase, run, setup, assert_equal, turtle
+import mock
+from testify import TestCase, run, setup, assert_equal
+from testify import setup_teardown
+from tron.commands import display
 
-from tron.commands.display import Color, DisplayServices, DisplayJobRuns
+from tron.commands.display import DisplayServices, DisplayJobRuns
 from tron.commands.display import DisplayActionRuns, DisplayJobs
+from tron.core import actionrun, service
 
 
 class DisplayServicesTestCase(TestCase):
 
     @setup
     def setup_data(self):
-        Color.enabled = True
         self.data = [
             dict(name="My Service",      state="stopped", live_count="4", enabled=True),
             dict(name="Another Service", state="running", live_count="2", enabled=False),
             dict(name="Yet another",     state="running", live_count="1", enabled=True)
         ]
-        self.display = DisplayServices(80)
+        self.display = DisplayServices()
 
     def test_format(self):
         out = self.display.format(self.data)
@@ -35,7 +38,7 @@ class DisplayJobRunsTestCase(TestCase):
     def setup_data(self):
         self.data = [
             dict(
-                id='something.23', state='FAIL', node='machine4',
+                id='something.23', state='FAIL', node=mock.MagicMock(),
                 run_num=23,
                 run_time='2012-01-20 23:11:23',
                 start_time='2012-01-20 23:11:23',
@@ -44,7 +47,7 @@ class DisplayJobRunsTestCase(TestCase):
                 manual=False,
             ),
             dict(
-                id='something.55', state='QUE', node='machine3',
+                id='something.55', state='QUE', node=mock.MagicMock(),
                 run_num=55,
                 run_time='2012-01-20 23:11:23',
                 start_time='2012-01-20 23:11:23',
@@ -58,7 +61,7 @@ class DisplayJobRunsTestCase(TestCase):
             id='something.23.other',
             name='other',
             state='FAIL',
-            node='machine4',
+            node=mock.MagicMock(),
             command='echo 123',
             raw_command='echo 123',
             run_time='2012-01-20 23:11:23',
@@ -69,38 +72,22 @@ class DisplayJobRunsTestCase(TestCase):
             stderr=[]
         )
 
-        Color.enabled = True
-        self.options = turtle.Turtle(warn=False, num_displays=4)
 
     def test_format(self):
-        display = DisplayJobRuns(options=self.options)
-        out = display.format(self.data)
+        out = DisplayJobRuns().format(self.data)
         lines = out.split('\n')
         assert_equal(len(lines), 7)
 
-    def test_format_with_warn(self):
-        self.options.warn = True
-        self.data = self.data[:1]
-        self.data[0]['runs'] = [self.action_run]
-
-        display = DisplayJobRuns(options=self.options)
-        out = display.format(self.data)
-        lines = out.split('\n')
-        assert_equal(len(lines), 16)
-        assert lines[13].startswith('Actions:'), lines[13]
-
 
 class DisplayJobsTestCase(TestCase):
 
     @setup
     def setup_data(self):
-        Color.enabled = True
-        self.options = turtle.Turtle(warn=False, num_displays=4)
         self.data = [
             dict(name='important_things', status='running',
-                scheduler='DailyJob', last_success='unknown'),
+                scheduler=mock.MagicMock(), last_success='unknown'),
             dict(name='other_thing', status='success',
-                scheduler='DailyJob', last_success='2012-01-23 10:23:23',
+                scheduler=mock.MagicMock(), last_success='2012-01-23 10:23:23',
                 action_names=['other', 'first'],
                 node_pool=['blam']),
         ]
@@ -115,7 +102,7 @@ class DisplayJobsTestCase(TestCase):
                     id='something.23.other',
                     name='other',
                     state='FAIL',
-                    node='machine4',
+                    node=mock.MagicMock(),
                     command='echo 123',
                     raw_command='echo 123',
                     run_time='2012-01-20 23:11:23',
@@ -137,8 +124,7 @@ class DisplayJobsTestCase(TestCase):
         ]
 
     def do_format(self):
-        display = DisplayJobs(self.options).format(self.data)
-        out = display.format(self.data)
+        out = DisplayJobs().format(self.data)
         lines = out.split('\n')
         return lines
 
@@ -151,17 +137,10 @@ class DisplayActionsTestCase(TestCase):
 
     @setup
     def setup_data(self):
-        Color.enabled = True
-        self.options = turtle.Turtle(
-            warn=False,
-            num_displays=6,
-            stdout=False,
-            stderr=False
-        )
         self.data = {
             'id': 'something.23',
             'state': 'UNKWN',
-            'node': 'something',
+            'node': {'hostname': 'something', 'username': 'a'},
             'run_time': 'sometime',
             'start_time': 'sometime',
             'end_time': 'sometime',
@@ -205,20 +184,65 @@ class DisplayActionsTestCase(TestCase):
         }
 
     def format_lines(self):
-        display = DisplayActionRuns(options=self.options)
-        out = display.format(self.data)
+        out = DisplayActionRuns().format(self.data)
         return out.split('\n')
 
     def test_format(self):
         lines = self.format_lines()
         assert_equal(len(lines), 13)
 
-    def test_format_warn(self):
-        self.data['runs'] = [self.data['runs'][2]]
-        self.data['runs'][0].update(self.details)
-        self.options.warn = True
-        lines = self.format_lines()
-        assert_equal(len(lines), 11)
+
+class AddColorForStateTestCase(TestCase):
+
+    @setup_teardown
+    def enable_color(self):
+        with display.Color.enable():
+            yield
+
+    def test_add_red(self):
+        text = display.add_color_for_state(actionrun.ActionRun.STATE_FAILED.name)
+        assert text.startswith(display.Color.colors['red']), text
+
+    def test_add_green(self):
+        text = display.add_color_for_state(actionrun.ActionRun.STATE_RUNNING.name)
+        assert text.startswith(display.Color.colors['green']), text
+
+    def test_add_blue(self):
+        text = display.add_color_for_state(service.ServiceState.DISABLED)
+        assert text.startswith(display.Color.colors['blue']), text
+
+
+class DisplayNodeTestCase(TestCase):
+
+    node_source = {
+        'name': 'name',
+        'hostname': 'hostname',
+        'username': 'username'}
+
+    def test_display_node(self):
+        result = display.display_node(self.node_source)
+        assert_equal(result, 'username@hostname')
+
+    def test_display_node_pool(self):
+        source = {'name': 'name', 'nodes': [self.node_source]}
+        result = display.display_node_pool(source)
+        assert_equal(result, 'name (1 node(s))')
+
+class DisplaySchedulerTestCase(TestCase):
+
+    def test_display_scheduler_no_jitter(self):
+        source = {'value': '5 minutes', 'type': 'interval', 'jitter': ''}
+        result = display.display_scheduler(source)
+        assert_equal(result, 'interval 5 minutes')
+
+    def test_display_scheduler_with_jitter(self):
+        source = {
+            'value': '5 minutes',
+            'type': 'interval',
+            'jitter': ' (+/- 2 min)'}
+        result = display.display_scheduler(source)
+        assert_equal(result, 'interval 5 minutes%s' % (source['jitter']))
+
 
 
 if __name__ == "__main__":
