@@ -153,6 +153,7 @@ services:
 
     def test_attributes(self):
         expected = schema.TronConfig(
+            action_runner=FrozenDict(),
             output_stream_dir='/tmp',
             command_context=FrozenDict({
                 'python': '/usr/bin/python',
@@ -162,15 +163,20 @@ services:
                 agent=False,
                 identities=('tests/test_id_rsa',),
                 known_hosts_file=None,
+                connect_timeout=30,
+                idle_connection_timeout=3600,
+                jitter_min_load=4,
+                jitter_max_delay=20,
+                jitter_load_factor=1,
             ),
             notification_options=None,
             time_zone=pytz.timezone("EST"),
             state_persistence=config_parse.DEFAULT_STATE_PERSISTENCE,
             nodes=FrozenDict({
                 'node0': schema.ConfigNode(name='node0',
-                    username=os.environ['USER'], hostname='node0'),
+                    username=os.environ['USER'], hostname='node0', port=22),
                 'node1': schema.ConfigNode(name='node1',
-                    username=os.environ['USER'], hostname='node1')
+                    username=os.environ['USER'], hostname='node1', port=22)
             }),
             node_pools=FrozenDict({
                 'nodePool': schema.ConfigNodePool(nodes=('node0', 'node1'),
@@ -182,7 +188,7 @@ services:
                     namespace='MASTER',
                     node='node0',
                     schedule=ConfigIntervalScheduler(
-                        timedelta=datetime.timedelta(0, 20)),
+                        timedelta=datetime.timedelta(0, 20), jitter=None),
                     actions=FrozenDict({
                         'action0_0': schema.ConfigAction(
                             name='action0_0',
@@ -198,6 +204,7 @@ services:
                         command='test_command0.1',
                         node=None),
                     enabled=True,
+                    max_runtime=None,
                     allow_overlap=False),
                 'MASTER.test_job1': schema.ConfigJob(
                     name='MASTER.test_job1',
@@ -208,6 +215,7 @@ services:
                         days=set([1, 3, 5]),
                         hour=0, minute=30, second=0,
                         original="00:30:00 MWF",
+                        jitter=None,
                     ),
                     actions=FrozenDict({
                         'action1_1': schema.ConfigAction(
@@ -225,6 +233,7 @@ services:
                     run_limit=50,
                     all_nodes=False,
                     cleanup_action=None,
+                    max_runtime=None,
                     allow_overlap=True),
                 'MASTER.test_job2': schema.ConfigJob(
                     name='MASTER.test_job2',
@@ -235,6 +244,7 @@ services:
                         days=set(),
                         hour=16, minute=30, second=0,
                         original="16:30:00 ",
+                        jitter=None,
                     ),
                     actions=FrozenDict({
                         'action2_0': schema.ConfigAction(
@@ -247,6 +257,7 @@ services:
                     run_limit=50,
                     all_nodes=False,
                     cleanup_action=None,
+                    max_runtime=None,
                     allow_overlap=False),
                 'MASTER.test_job3': schema.ConfigJob(
                     name='MASTER.test_job3',
@@ -275,6 +286,7 @@ services:
                     run_limit=50,
                     all_nodes=False,
                     cleanup_action=None,
+                    max_runtime=None,
                     allow_overlap=False),
                 'MASTER.test_job4': schema.ConfigJob(
                     name='MASTER.test_job4',
@@ -283,7 +295,8 @@ services:
                     schedule=schedule_parse.ConfigDailyScheduler(
                         days=set(),
                         hour=0, minute=0, second=0,
-                        original='00:00:00 '
+                        original='00:00:00 ',
+                        jitter=None,
                     ),
                     actions=FrozenDict({
                         'action4_0': schema.ConfigAction(
@@ -296,6 +309,7 @@ services:
                     all_nodes=True,
                     cleanup_action=None,
                     enabled=False,
+                    max_runtime=None,
                     allow_overlap=False)
                 }),
                 services=FrozenDict({
@@ -306,7 +320,7 @@ services:
                         pid_file='/var/run/%(name)s-%(instance_number)s.pid',
                         command='service_command0',
                         monitor_interval=20,
-                        restart_interval=None,
+                        restart_delay=None,
                         count=2)
                 }
             )
@@ -329,10 +343,10 @@ services:
         assert_equal(test_config, expected)
         assert_equal(test_config.jobs['MASTER.test_job4'].enabled, False)
 
-
     def test_empty_node_test(self):
         valid_config_from_yaml("""nodes:""")
 
+
 class NamedConfigTestCase(TestCase):
     config = """
 jobs:
@@ -416,7 +430,9 @@ services:
                     namespace='test_namespace',
                     node='node0',
                     schedule=ConfigIntervalScheduler(
-                        timedelta=datetime.timedelta(0, 20)),
+                        timedelta=datetime.timedelta(0, 20),
+                        jitter=None,
+                    ),
                     actions=FrozenDict({
                         'action0_0': schema.ConfigAction(
                             name='action0_0',
@@ -432,6 +448,7 @@ services:
                         command='test_command0.1',
                         node=None),
                     enabled=True,
+                    max_runtime=None,
                     allow_overlap=False),
                 'test_job1': schema.ConfigJob(
                     name='test_job1',
@@ -444,6 +461,7 @@ services:
                         minute=30,
                         second=0,
                         original="00:30:00 MWF",
+                        jitter=None,
                     ),
                     actions=FrozenDict({
                         'action1_1': schema.ConfigAction(
@@ -461,6 +479,7 @@ services:
                     run_limit=50,
                     all_nodes=False,
                     cleanup_action=None,
+                    max_runtime=None,
                     allow_overlap=True),
                 'test_job2': schema.ConfigJob(
                     name='test_job2',
@@ -473,6 +492,7 @@ services:
                         minute=30,
                         second=0,
                         original="16:30:00 ",
+                        jitter=None,
                     ),
                     actions=FrozenDict({
                         'action2_0': schema.ConfigAction(
@@ -485,6 +505,7 @@ services:
                     run_limit=50,
                     all_nodes=False,
                     cleanup_action=None,
+                    max_runtime=None,
                     allow_overlap=False),
                 'test_job3': schema.ConfigJob(
                     name='test_job3',
@@ -513,6 +534,7 @@ services:
                     run_limit=50,
                     all_nodes=False,
                     cleanup_action=None,
+                    max_runtime=None,
                     allow_overlap=False),
                 'test_job4': schema.ConfigJob(
                     name='test_job4',
@@ -522,6 +544,7 @@ services:
                         days=set(),
                         hour=0, minute=0, second=0,
                         original="00:00:00 ",
+                        jitter=None,
                     ),
                     actions=FrozenDict({
                         'action4_0': schema.ConfigAction(
@@ -534,6 +557,7 @@ services:
                     all_nodes=True,
                     cleanup_action=None,
                     enabled=False,
+                    max_runtime=None,
                     allow_overlap=False)
                 }),
                 services=FrozenDict({
@@ -544,7 +568,7 @@ services:
                         pid_file='/var/run/%(name)s-%(instance_number)s.pid',
                         command='service_command0',
                         monitor_interval=20,
-                        restart_interval=None,
+                        restart_delay=None,
                         count=2)
                 }
             )
@@ -892,7 +916,8 @@ class ValidateJobsAndServicesTestCase(TestCase):
             schema.ConfigJob(name='MASTER.test_job0',
                 namespace='MASTER',
                 node='node0',
-                schedule=ConfigIntervalScheduler(timedelta=datetime.timedelta(0, 20)),
+                schedule=ConfigIntervalScheduler(
+                    timedelta=datetime.timedelta(0, 20), jitter=None),
                 actions=FrozenDict({'action0_0':
                       schema.ConfigAction(name='action0_0',
                                    command='test_command0.0',
@@ -905,7 +930,8 @@ class ValidateJobsAndServicesTestCase(TestCase):
                      name='cleanup',
                      node=None),
                 enabled=True,
-                allow_overlap=False)
+                allow_overlap=False,
+                max_runtime=None)
             }
 
         expected_services = {'MASTER.test_service0':
@@ -915,7 +941,7 @@ class ValidateJobsAndServicesTestCase(TestCase):
                           pid_file='/var/run/%(name)s-%(instance_number)s.pid',
                           command='service_command0',
                           monitor_interval=20,
-                          restart_interval=None,
+                          restart_delay=None,
                           count=2)
             }
 
@@ -926,32 +952,6 @@ class ValidateJobsAndServicesTestCase(TestCase):
         assert_equal(expected_services, config['services'])
 
 
-StubConfigObject = schema.config_object_factory(
-    'StubConfigObject',
-    ['req1', 'req2'],
-    ['opt1', 'opt2']
-)
-
-class StubValidator(config_parse.Validator):
-    config_class = StubConfigObject
-
-class ValidatorTestCase(TestCase):
-
-    @setup
-    def setup_validator(self):
-        self.validator = StubValidator()
-
-    def test_validate_with_none(self):
-        expected_msg = "A StubObject is required"
-        exception = assert_raises(ConfigError,
-            self.validator.validate, None, NullConfigContext)
-        assert_in(expected_msg, str(exception))
-
-    def test_validate_optional_with_none(self):
-        self.validator.optional = True
-        assert_equal(self.validator.validate(None, NullConfigContext), None)
-
-
 class ValidCleanupActionNameTestCase(TestCase):
 
     def test_valid_cleanup_action_name_pass(self):
@@ -1008,7 +1008,9 @@ class BuildFormatStringValidatorTestCase(TestCase):
 
     def test_validator_error(self):
         template = "The %(one)s thing I %(seven)s is %(unknown)s"
-        assert_raises(ConfigError, self.validator, template, NullConfigContext)
+        exception = assert_raises(ConfigError,
+            self.validator, template, NullConfigContext)
+        assert_in("Unknown context variable 'unknown'", str(exception))
 
     def test_validator_passes_with_context(self):
         template = "The %(one)s thing I %(seven)s is %(mars)s"
@@ -1088,6 +1090,16 @@ class ConfigContainerTestCase(TestCase):
         assert_equal(node_names, expected)
 
 
+class ValidateServiceTestCase(TestCase):
+
+    def test_cast_restart_interval_deprecation(self):
+        config = {'restart_interval': 50.0}
+        context = config_utils.NullConfigContext
+        casted_config = config_parse.ValidateService().cast(config, context)
+        expected = {'restart_delay': 50.0, 'namespace': schema.MASTER_NAMESPACE}
+        assert_equal(casted_config, expected)
+
+
 class ValidateSSHOptionsTestCase(TestCase):
 
     @setup
