@@ -145,9 +145,12 @@ class ServiceInstanceMonitorTaskTestCase(TestCase):
 
     def test_fail(self):
         self.task.action = mock.create_autospec(actioncommand.ActionCommand)
+        original_action = self.task.action
         self.task.fail()
         self.task.notify.assert_called_with(self.task.NOTIFY_FAILED)
+        self.task.node.stop.assert_called_with(original_action)
         assert_equal(self.task.action, actioncommand.CompletedActionCommand)
+        assert_equal(original_action.write_stderr.call_count, 1)
 
 
 class ServiceInstanceStopTaskTestCase(TestCase):
@@ -162,7 +165,7 @@ class ServiceInstanceStopTaskTestCase(TestCase):
         autospec_method(self.task.notify)
 
     def test_kill_task(self):
-        assert self.task.kill()
+        assert self.task.stop()
 
     def test_handle_action_event_complete(self):
         action = mock.create_autospec(ActionCommand)
@@ -283,7 +286,7 @@ class ServiceInstanceTestCase(TestCase):
 
     def test_stop(self):
         self.instance.stop()
-        self.instance.stop_task.kill.assert_called_with()
+        self.instance.stop_task.stop.assert_called_with()
         self.instance.machine.transition.assert_called_with('stop')
         self.instance.monitor_task.cancel.assert_called_with()
 
@@ -314,7 +317,7 @@ class ServiceInstanceTestCase(TestCase):
 
     def test_handle_start_task_complete_from_unknown(self):
         self.instance._handle_start_task_complete()
-        self.instance.stop_task.kill.assert_called_with()
+        self.instance.stop_task.stop.assert_called_with()
 
     def test_state_data(self):
         expected = {
