@@ -1,4 +1,6 @@
 import logging
+import os
+from tron.config import schema
 from tron.serialize import filehandler
 
 from tron.utils import state, timeutils
@@ -16,6 +18,10 @@ class CompletedActionCommand(object):
     is_done = True
     is_failed = False
 
+    @staticmethod
+    def write_stderr(_):
+        pass
+
 
 class ActionCommand(object):
     """An ActionCommand encapsulates a runnable task that is passed to a node
@@ -40,8 +46,7 @@ class ActionCommand(object):
     def __init__(self, id, command, serializer=None):
         self.id             = id
         self.command        = command
-        self.machine        = state.StateMachine(
-                                initial_state=self.PENDING, delegate=self)
+        self.machine        = state.StateMachine(self.PENDING, delegate=self)
         self.exit_status    = None
         self.start_time     = None
         self.end_time       = None
@@ -144,3 +149,71 @@ class StringBufferStore(object):
 
     def clear(self):
         self.buffers.clear()
+
+
+class NoActionRunnerFactory(object):
+    """Action runner factory that does not wrap the action run command."""
+
+    @classmethod
+    def create(cls, id, command, serializer):
+        return ActionCommand(id, command, serializer)
+
+    @classmethod
+    def build_stop_action_command(cls, _id, _command):
+        """It is not possible to stop action commands without a runner."""
+        raise NotImplementedError("An action_runner is required to stop.")
+
+
+class SubprocessActionRunnerFactory(object):
+    """Run actions by wrapping them in `action_runner.py`."""
+
+    runner_exec_name =  "action_runner.py"
+    status_exec_name =  "action_status.py"
+
+    def __init__(self, status_path, exec_path):
+        self.status_path = status_path
+        self.exec_path = exec_path
+
+    @classmethod
+    def from_config(cls, config):
+        return cls(config.remote_status_path, config.remote_exec_path)
+
+    def create(self, id, command, serializer):
+        command = self.build_command(id, command, self.runner_exec_name)
+        return ActionCommand(id, command, serializer)
+
+    def build_command(self, id, command, exec_name):
+        status_path = os.path.join(self.status_path, id)
+        runner_path = os.path.join(self.exec_path, exec_name)
+        return '''%s "%s" "%s"''' % (runner_path, status_path, command)
+
+    def build_stop_action_command(self, id, command):
+        command = self.build_command(id, command, self.status_exec_name)
+        run_id = '%s.%s' % (id, command)
+        return ActionCommand(run_id, command, StringBufferStore())
+
+    def __eq__(self, other):
+        return (self.__class__ == other.__class__ and
+            self.status_path == other.status_path and
+            self.exec_path == other.exec_path)
+
+    def __ne__(self, other):
+        return not self == other
+
+
+def create_action_runner_factory_from_config(config):
+    """A factory-factory method which returns a callable that can be used to
+    create ActionCommand objects. The factory definition should match the
+    constructor for ActionCommand.
+    """
+    if not config:
+        return NoActionRunnerFactory
+
+    if config.runner_type not in schema.ActionRunnerTypes:
+        raise ValueError("Unknown runner type: %s", config.runner_type)
+
+    if config.runner_type == schema.ActionRunnerTypes.none:
+        return NoActionRunnerFactory
+
+    if config.runner_type == schema.ActionRunnerTypes.subprocess:
+        return SubprocessActionRunnerFactory.from_config(config)
