@@ -1,8 +1,10 @@
 """
 Format and color output for tron commands.
 """
+import contextlib
 from functools import partial
 from operator import itemgetter
+from tron.core import actionrun, job, service
 
 
 class Color(object):
@@ -27,12 +29,26 @@ class Color(object):
         'end':                  '\033[0m',
     }
 
+    @classmethod
+    @contextlib.contextmanager
+    def enable(cls):
+        old_val = cls.enabled
+        try:
+            cls.enabled = True
+            yield
+        finally:
+            cls.enabled = old_val
+
     @classmethod
     def set(cls, color_name, text):
         if not cls.enabled or not color_name:
             return unicode(text)
         return cls.colors[color_name.lower()] + unicode(text) + cls.colors['end']
 
+    @classmethod
+    def toggle(cls, enable):
+        cls.enabled = enable
+
 
 class TableDisplay(object):
     """Base class for displaying columns of data.  This class takes a list
@@ -64,12 +80,12 @@ class TableDisplay(object):
     colors = None
     title = None
     resize_fields = set()
+    reversed = False
 
     header_color = 'hgray'
 
-    def __init__(self, options=None):
+    def __init__(self):
         self.out = []
-        self.options = options
 
     def banner(self):
         if not self.title:
@@ -125,7 +141,8 @@ class TableDisplay(object):
         return None
 
     def rows(self):
-        return sorted(self.data, key=itemgetter(self.fields[0]))
+        return sorted(self.data,
+            key=itemgetter(self.fields[0]), reverse=self.reversed)
 
     def store_data(self, data):
         self.data = data
@@ -164,11 +181,16 @@ class TableDisplay(object):
 
 
 def add_color_for_state(state):
-    if state.upper() in ('FAILED', 'FAIL'):
+    if state == actionrun.ActionRun.STATE_FAILED.name:
         return Color.set('red', state)
-    if state.upper() in ('UP', 'ENABLED', 'RUNNING', 'SUCC'):
+    if state in set((
+        actionrun.ActionRun.STATE_RUNNING.name,
+        actionrun.ActionRun.STATE_SUCCEEDED.name,
+        job.Job.STATUS_ENABLED,
+        service.ServiceState.UP
+    )):
         return Color.set('green', state)
-    if state.upper() in ('DISABLED', 'DOWN'):
+    if state in set((job.Job.STATUS_DISABLED, service.ServiceState.DISABLED)):
         return Color.set('blue', state)
     return state
 
@@ -180,9 +202,14 @@ def format_fields(display_obj, content):
             return field_value
         return display_obj.colors[field](field_value)
 
+    def format_field(field):
+        value = content[field]
+        if value is None:
+            return ''
+        return field_display_mapping.get(field, lambda f: f)(value)
+
     def build_field(label, field):
-        field_value = '' if content[field] is None else content[field]
-        return "%-20s: %s" % (label, add_color(field, field_value))
+        return "%-20s: %s" % (label, add_color(field, format_field(field)))
 
     return "\n".join(build_field(*item) for item in display_obj.detail_labels)
 
@@ -201,7 +228,8 @@ def format_service_details(service_content):
         def format(inst):
             state = add_color_for_state(inst['state'])
             failures = get_failure_messages(inst['failures'])
-            return format_str % (inst['id'], inst['node'], state, failures)
+            node = display_node(inst['node'])
+            return format_str % (inst['id'], node, state, failures)
         return [format(instance) for instance in service_instances]
 
     details     = format_fields(DisplayServices, service_content)
@@ -209,9 +237,9 @@ def format_service_details(service_content):
     return details + '\n\nInstances:\n' + '\n'.join(instances)
 
 
-def format_job_details(job_content, options):
+def format_job_details(job_content):
     details = format_fields(DisplayJobs, job_content)
-    job_runs = DisplayJobRuns(options).format(job_content['runs'])
+    job_runs = DisplayJobRuns().format(job_content['runs'])
     actions = "\n\nList of Actions:\n%s" % '\n'.join(job_content['action_names'])
     return details + actions + "\n" + job_runs
 
@@ -244,8 +272,8 @@ class DisplayServices(TableDisplay):
         ('Command',             'command'           ),
         ('Pid Filename',        'pid_filename'      ),
         ('Node Pool',           'node_pool'         ),
-        ('Restart interval',    'restart_interval'  ),
         ('Monitor interval',    'monitor_interval'  ),
+        ('Restart delay',       'restart_delay'     ),
     ]
 
     colors = {
@@ -259,8 +287,9 @@ class DisplayJobRuns(TableDisplay):
 
     columns = ['Run ID',    'State',    'Node', 'Scheduled Time']
     fields  = ['run_num',   'state',    'node', 'run_time']
-    widths  = [10,          6,          30,     25              ]
+    widths  = [10,          12,         30,     25]
     title = 'job runs'
+    reversed = True
 
     detail_labels = [
         ('Job Run',             'id'),
@@ -278,13 +307,6 @@ class DisplayJobRuns(TableDisplay):
         'manual':    lambda value: Color.set('cyan' if value else None, value),
     }
 
-    def rows(self):
-        data_rows = self.data
-        if self.options.warn:
-            warn_only_func = lambda r: r['state'] in ['FAIL', 'UNKWN', 'QUE']
-            data_rows = filter(warn_only_func, self.data)
-        return data_rows[:self.options.num_displays]
-
     def format_value(self, field_idx, value):
         if self.fields[field_idx] == 'run_num':
             value = '.' + str(value)
@@ -292,6 +314,9 @@ class DisplayJobRuns(TableDisplay):
         if self.fields[field_idx] == 'scheduled_time':
             value = value or '-'
 
+        if self.fields[field_idx] == 'node':
+            value = display_node(value)
+
         return super(DisplayJobRuns, self).format_value(field_idx, value)
 
     def row_color(self, fields):
@@ -306,10 +331,6 @@ class DisplayJobRuns(TableDisplay):
             ' ' * self.widths[0], start, end, duration)
         self.out.append(Color.set('gray', row_data))
 
-        if self.options.warn:
-            display_action = DisplayActionRuns(self.options)
-            self.out.append(display_action.format(row))
-
 
 class DisplayJobs(TableDisplay):
 
@@ -323,6 +344,7 @@ class DisplayJobs(TableDisplay):
         ('Job',                 'name'              ),
         ('State',               'status'            ),
         ('Scheduler',           'scheduler'         ),
+        ('Max runtime',         'max_runtime'       ),
         ('Node Pool',           'node_pool'         ),
         ('Run on all nodes',    'all_nodes'         ),
         ('Allow overlapping',   'allow_overlap'     ),
@@ -334,12 +356,18 @@ class DisplayJobs(TableDisplay):
         'status':    add_color_for_state
     }
 
+    def format_value(self, field_idx, value):
+        if self.fields[field_idx] == 'scheduler':
+            value = display_scheduler(value)
+
+        return super(DisplayJobs, self).format_value(field_idx, value)
+
 
 class DisplayActionRuns(TableDisplay):
 
     columns = ['Action', 'State', 'Start Time', 'End Time', 'Duration']
     fields  = ['id',     'state', 'start_time', 'end_time', 'duration']
-    widths  = [40,         7,        22,          22,         10      ]
+    widths  = [40,       12,      22,           22,         10     ]
     title = 'actions'
     resize_fields = ['id']
 
@@ -361,14 +389,9 @@ class DisplayActionRuns(TableDisplay):
     }
 
     def banner(self):
-        if self.options.display_preface:
-            self.out.append(format_fields(DisplayJobRuns, self.job_run))
+        self.out.append(format_fields(DisplayJobRuns, self.job_run))
         super(DisplayActionRuns, self).banner()
 
-    def footer(self):
-        if len(self.rows()) < len(self.data) and not self.options.warn:
-            self.out.append('...')
-
     def format_value(self, field_idx, value):
         if self.fields[field_idx] == 'id':
             value = '.' + value.rsplit('.', 1)[-1]
@@ -387,14 +410,6 @@ class DisplayActionRuns(TableDisplay):
         self.data = data['runs']
         self.job_run = data
 
-    def rows(self):
-        data_rows = self.data
-        if self.options.warn:
-            warn_only_func = lambda r: r['state'] in ['FAIL', 'UNKWN', 'QUE']
-            data_rows = filter(warn_only_func, self.data)
-        return data_rows[:self.options.num_displays]
-
-
 
 class DisplayEvents(TableDisplay):
 
@@ -405,6 +420,25 @@ class DisplayEvents(TableDisplay):
     resize_fields = ['entity']
 
 
+def display_node(source):
+    return '%s@%s' % (source['username'], source['hostname'])
+
+
+def display_node_pool(source):
+    return "%s (%d node(s))" % (source['name'], len(source['nodes']))
+
+
+def display_scheduler(source):
+    return "%s %s%s" % (source['type'], source['value'], source['jitter'])
+
+
+field_display_mapping = {
+    'node':             display_node,
+    'node_pool':        display_node_pool,
+    'scheduler':        display_scheduler,
+}
+
+
 def view_with_less(content, color=True):
     """Send `content` through less."""
     import subprocess
@@ -415,4 +449,4 @@ def view_with_less(content, color=True):
     less_proc = subprocess.Popen(cmd, stdin=subprocess.PIPE)
     less_proc.stdin.write(content)
     less_proc.stdin.close()
-    less_proc.wait()
\ No newline at end of file
+    less_proc.wait()
