@@ -9,8 +9,8 @@ import os
 import pytz
 from tron import command_context
 
-from tron.config import ConfigError, config_utils
-from tron.config.config_utils import NullConfigContext, ConfigContext
+from tron.config import ConfigError, config_utils, schema
+from tron.config.config_utils import ConfigContext, Validator
 from tron.config.config_utils import valid_string, valid_bool
 from tron.config.config_utils import valid_identifier
 from tron.config.config_utils import build_list_of_type_validator
@@ -19,9 +19,10 @@ from tron.config.config_utils import build_dict_name_validator
 from tron.config.config_utils import valid_int, valid_float, valid_dict
 from tron.config.config_utils import PartialConfigContext
 from tron.config.schedule_parse import valid_schedule
-from tron.config.schema import TronConfig, NamedTronConfig, NotificationOptions, CLEANUP_ACTION_NAME
+from tron.config.schema import TronConfig, NamedTronConfig, NotificationOptions
+from tron.config.schema import CLEANUP_ACTION_NAME
 from tron.config.schema import ConfigSSHOptions
-from tron.config.schema import ConfigNode, ConfigNodePool, ConfigState
+from tron.config.schema import ConfigState
 from tron.config.schema import ConfigJob, ConfigAction, ConfigCleanupAction
 from tron.config.schema import ConfigService
 from tron.config.schema import MASTER_NAMESPACE
@@ -45,130 +46,13 @@ def build_format_string_validator(context_object):
         try:
             value % context
             return value
-        except (KeyError, ValueError):
-            error_msg = "Invalid template string at %s: %s"
-            raise ConfigError(error_msg % (config_context.path, value))
+        except (KeyError, ValueError), e:
+            error_msg = "Unknown context variable %s at %s: %s"
+            raise ConfigError(error_msg % (e, config_context.path, value))
 
     return validator
 
 
-
-# TODO: extract code
-class Validator(object):
-    """Base class for validating a collection and creating a mutable
-    collection from the source.
-    """
-    config_class            = None
-    defaults                = {}
-    validators              = {}
-    optional                = False
-
-    def validate(self, in_dict, config_context):
-        if self.optional and in_dict is None:
-            return None
-
-        if in_dict is None:
-            raise ConfigError("A %s is required." % self.type_name)
-
-        shortcut_value = self.do_shortcut(in_dict)
-        if shortcut_value:
-            return shortcut_value
-
-        config_context = self.build_context(in_dict, config_context)
-        in_dict = self.cast(in_dict, config_context)
-        self.validate_required_keys(in_dict)
-        self.validate_extra_keys(in_dict)
-        return self.build_config(in_dict, config_context)
-
-    def __call__(self, in_dict, config_context=NullConfigContext):
-        return self.validate(in_dict, config_context)
-
-    @property
-    def type_name(self):
-        """Return a string that represents the config_class being validated.
-        This name is used for error messages, so we strip off the word
-        Config so the name better matches what the user sees in the config.
-        """
-        return self.config_class.__name__.replace("Config", "")
-
-    def do_shortcut(self, in_dict):
-        """Override if your validator can skip most of the validation by
-        checking this condition.  If this returns a truthy value, the
-        validation will end immediately and return that value.
-        """
-        pass
-
-    def cast(self, in_dict, _):
-        """If your validator accepts input in different formations, override
-        this method to cast your input into a common format.
-        """
-        return in_dict
-
-    def build_context(self, in_dict, config_context):
-        path = self.path_name(in_dict.get('name'))
-        return config_context.build_child_context(path)
-
-    def validate_required_keys(self, in_dict):
-        """Check that all required keys are present."""
-        missing_keys = set(self.config_class.required_keys) - set(in_dict)
-        if not missing_keys:
-            return
-
-        keys = self.config_class.required_keys + self.config_class.optional_keys
-        missing_key_str = ', '.join(missing_keys)
-        if 'name' in keys and 'name' in in_dict:
-            msg  = "%s %s is missing options: %s"
-            name = in_dict['name']
-            raise ConfigError(msg % (self.type_name, name, missing_key_str))
-
-        msg = "Nameless %s is missing options: %s"
-        raise ConfigError(msg % (self.type_name, missing_key_str))
-
-    def validate_extra_keys(self, in_dict):
-        """Check that no unexpected keys are present."""
-        conf_class      = self.config_class
-        all_keys        = conf_class.required_keys + conf_class.optional_keys
-        extra_keys      = set(in_dict) - set(all_keys)
-        if not extra_keys:
-            return
-
-        msg             = "Unknown keys in %s %s: %s"
-        name            = in_dict.get('name', '')
-        raise ConfigError(msg % (self.type_name, name, ', '.join(extra_keys)))
-
-    def set_defaults(self, output_dict, _config_context):
-        """Set any default values for any optional values that were not
-        specified.
-        """
-        for key, value in self.defaults.iteritems():
-            if key not in output_dict:
-                output_dict[key] = value
-
-    def path_name(self, name=None):
-        return '%s.%s' % (self.type_name, name) if name else self.type_name
-
-    def post_validation(self, valid_input, config_context):
-        """Perform additional validation."""
-        pass
-
-    def build_config(self, in_dict, config_context):
-        output_dict = self.validate_contents(in_dict, config_context)
-        self.post_validation(output_dict, config_context)
-        self.set_defaults(output_dict, config_context)
-        return self.config_class(**output_dict)
-
-    def validate_contents(self, input, config_context):
-        """Override this to validate each value in the input."""
-        valid_input = {}
-        for key, value in input.iteritems():
-            if key in self.validators:
-                child_context = config_context.build_child_context(key)
-                valid_input[key] = self.validators[key](value, child_context)
-            else:
-                valid_input[key] = value
-        return valid_input
-
-
 def valid_output_stream_dir(output_dir, config_context):
     """Returns a valid string for the output directory, or raises ConfigError
     if the output_dir is not valid.
@@ -243,19 +127,29 @@ def valid_node_name(value, config_context):
 
 class ValidateSSHOptions(Validator):
     """Validate SSH options."""
-    config_class =              ConfigSSHOptions
-    optional =                  True
+    config_class =                  ConfigSSHOptions
+    optional =                      True
     defaults = {
-        'agent':                False,
-        'identities':           (),
-        'known_hosts_file':     None,
+        'agent':                    False,
+        'identities':               (),
+        'known_hosts_file':         None,
+        'connect_timeout':          30,
+        'idle_connection_timeout':  3600,
+        'jitter_min_load':          4,
+        'jitter_max_delay':         20,
+        'jitter_load_factor':       1,
     }
 
     validators = {
-        'agent':                valid_bool,
-        'identities':           build_list_of_type_validator(
-                                    valid_identity_file, allow_empty=True),
-        'known_hosts_file':     valid_known_hosts_file
+        'agent':                    valid_bool,
+        'identities':               build_list_of_type_validator(
+                                        valid_identity_file, allow_empty=True),
+        'known_hosts_file':         valid_known_hosts_file,
+        'connect_timeout':          config_utils.valid_int,
+        'idle_connection_timeout':  config_utils.valid_int,
+        'jitter_min_load':          config_utils.valid_int,
+        'jitter_max_delay':         config_utils.valid_int,
+        'jitter_load_factor':       config_utils.valid_int,
     }
 
     def post_validation(self, valid_input, config_context):
@@ -278,30 +172,33 @@ valid_notification_options = ValidateNotificationOptions()
 
 
 class ValidateNode(Validator):
-    config_class =              ConfigNode
+    config_class =              schema.ConfigNode
     validators = {
-        'name':                 valid_identifier,
-        'username':             valid_string,
-        'hostname':             valid_string,
+        'name':                 config_utils.valid_identifier,
+        'username':             config_utils.valid_string,
+        'hostname':             config_utils.valid_string,
+        'port':                 config_utils.valid_int,
     }
 
-    DEFAULT_USER =              os.environ['USER']
+    defaults = {
+        'port':                 22,
+        'username':             os.environ['USER'],
+    }
 
     def do_shortcut(self, node):
         """Nodes can be specified with just a hostname string."""
         if isinstance(node, basestring):
-            return ConfigNode(
-                        hostname=node, name=node, username=self.DEFAULT_USER)
+            return schema.ConfigNode(hostname=node, name=node, **self.defaults)
 
-    def set_defaults(self, output_dict, _):
+    def set_defaults(self, output_dict, config_context):
+        super(ValidateNode, self).set_defaults(output_dict, config_context)
         output_dict.setdefault('name', output_dict['hostname'])
-        output_dict.setdefault('username', self.DEFAULT_USER)
 
 valid_node = ValidateNode()
 
 
 class ValidateNodePool(Validator):
-    config_class =              ConfigNodePool
+    config_class =              schema.ConfigNodePool
     validators = {
         'name':                 valid_identifier,
         'nodes':                build_list_of_type_validator(valid_identifier),
@@ -382,7 +279,8 @@ class ValidateJob(Validator):
         'cleanup_action':       None,
         'enabled':              True,
         'queueing':             True,
-        'allow_overlap':        False
+        'allow_overlap':        False,
+        'max_runtime':          None,
     }
 
     validators = {
@@ -396,6 +294,7 @@ class ValidateJob(Validator):
         'queueing':             valid_bool,
         'enabled':              valid_bool,
         'allow_overlap':        valid_bool,
+        'max_runtime':          config_utils.valid_time_delta,
     }
 
     def cast(self, in_dict, config_context):
@@ -444,7 +343,7 @@ class ValidateService(Validator):
 
     defaults = {
         'count':                1,
-        'restart_interval':     None
+        'restart_delay':        None
     }
 
     validators = {
@@ -454,18 +353,42 @@ class ValidateService(Validator):
         'monitor_interval':     valid_float,
         'count':                valid_int,
         'node':                 valid_node_name,
-        'restart_interval':     valid_float,
+        'restart_delay':        valid_float,
     }
 
     def cast(self, in_dict, config_context):
         in_dict['namespace'] = config_context.namespace
+
+        # TODO: Deprecated - remove in 0.7
+        if 'restart_interval' in in_dict:
+            msg = ("restart_interval at %s is deprecated. It has been renamed "
+                   "restart_delay and will be removed in 0.7")
+            log.warn(msg % config_context.path)
+            in_dict['restart_delay'] = in_dict.pop('restart_interval')
         return in_dict
 
 valid_service = ValidateService()
 
 
+class ValidateActionRunner(Validator):
+    config_class =              schema.ConfigActionRunner
+    optional =                  True
+    defaults = {
+        'runner_type':          None,
+        'remote_exec_path':     '',
+        'remote_status_path':   '/tmp',
+    }
+
+    validators = {
+        'runner_type':          config_utils.build_enum_validator(
+                                    schema.ActionRunnerTypes),
+        'remote_status_path':   valid_string,
+        'remote_exec_path':     valid_string,
+    }
+
+
 class ValidateStatePersistence(Validator):
-    config_class                = ConfigState
+    config_class                = schema.ConfigState
     defaults = {
         'buffer_size':          1,
         'connection_details':   None,
@@ -473,7 +396,8 @@ class ValidateStatePersistence(Validator):
 
     validators = {
         'name':                 valid_string,
-        'store_type':           valid_string,
+        'store_type':           config_utils.build_enum_validator(
+                                    schema.StatePersistenceTypes),
         'connection_details':   valid_string,
         'buffer_size':          valid_int,
     }
@@ -503,7 +427,7 @@ def validate_jobs_and_services(config, config_context):
 
 
 DEFAULT_STATE_PERSISTENCE = ConfigState('tron_state', 'shelve', None, 1)
-DEFAULT_NODE = ConfigNode('localhost', 'localhost', 'tronuser')
+DEFAULT_NODE = ValidateNode().do_shortcut('localhost')
 
 
 class ValidateConfig(Validator):
@@ -514,6 +438,7 @@ class ValidateConfig(Validator):
     """
     config_class =              TronConfig
     defaults = {
+        'action_runner':        {},
         'output_stream_dir':    None,
         'command_context':      {},
         'ssh_options':          ValidateSSHOptions.defaults,
@@ -528,6 +453,7 @@ class ValidateConfig(Validator):
     node_pools  = build_dict_name_validator(valid_node_pool, allow_empty=True)
     nodes       = build_dict_name_validator(valid_node, allow_empty=True)
     validators = {
+        'action_runner':        ValidateActionRunner(),
         'output_stream_dir':    valid_output_stream_dir,
         'command_context':      valid_command_context,
         'ssh_options':          valid_ssh_options,
