@@ -6,21 +6,29 @@ from collections import namedtuple
 import datetime
 import re
 
-from tron.config import ConfigError, config_utils
-from tron.utils import crontab, dicts
+from tron.config import ConfigError, config_utils, schema
+from tron.utils import crontab
 
 
+ConfigGenericSchedule = schema.config_object_factory(
+    'ConfigGenericSchedule',
+    ['type', 'value'],
+    ['jitter']
+)
+
 ConfigGrocScheduler = namedtuple('ConfigGrocScheduler',
-    'original ordinals weekdays monthdays months timestr')
+    'original ordinals weekdays monthdays months timestr jitter')
 
 ConfigCronScheduler = namedtuple('ConfigCronScheduler',
-    'original minutes hours monthdays months weekdays ordinals')
+    'original minutes hours monthdays months weekdays ordinals jitter')
 
 ConfigDailyScheduler    = namedtuple('ConfigDailyScheduler',
-    'original hour minute second days')
+    'original hour minute second days jitter')
 
 ConfigConstantScheduler = namedtuple('ConfigConstantScheduler', [])
-ConfigIntervalScheduler = namedtuple('ConfigIntervalScheduler', 'timedelta')
+
+ConfigIntervalScheduler = namedtuple('ConfigIntervalScheduler',
+    'timedelta jitter')
 
 
 class ScheduleParseError(ConfigError):
@@ -33,37 +41,60 @@ def pad_sequence(seq, size, padding=None):
     return (list(seq) + [padding for _ in xrange(size)])[:size]
 
 
+def schedule_config_from_string(schedule, config_context):
+    """Return a scheduler config object from a string."""
+    schedule = schedule.strip()
+    name, schedule_config = pad_sequence(schedule.split(None, 1), 2, padding='')
+    if name not in schedulers:
+        config = ConfigGenericSchedule('groc daily', schedule, jitter=None)
+        return parse_groc_expression(config, config_context)
+
+    config = ConfigGenericSchedule(name, schedule_config, jitter=None)
+    return validate_generic_schedule_config(config, config_context)
+
+
+def validate_generic_schedule_config(config, config_context):
+    return schedulers[config.type](config, config_context)
+
+
+# TODO: remove in 0.7
+def schedule_config_from_legacy_dict(schedule, config_context):
+    """Support old style schedules as dicts."""
+    if 'interval' in schedule:
+        config = ConfigGenericSchedule('interval', schedule['interval'], None)
+        return valid_interval_scheduler(config, config_context)
+
+    if 'start_time' in schedule or 'days' in schedule:
+        start_time = schedule.get('start_time', '00:00:00')
+        days = schedule.get('days', '')
+        scheduler_config = '%s %s' % (start_time, days)
+        config = ConfigGenericSchedule('daily', scheduler_config, None)
+        return valid_daily_scheduler(config, config_context)
+
+    path = config_context.path
+    raise ConfigError("Unknown scheduler at %s: %s" % (path, schedule))
+
+
 def valid_schedule(schedule, config_context):
     if isinstance(schedule, basestring):
-        schedule = schedule.strip()
-        scheduler_args = schedule.split()
-        scheduler_name = scheduler_args.pop(0).lower()
+        return schedule_config_from_string(schedule, config_context)
 
-        if schedule == 'constant':
-            return ConfigConstantScheduler()
-        elif scheduler_name == 'daily':
-            start_time, days = pad_sequence(scheduler_args, 2)
-            return valid_daily_scheduler(start_time, days, config_context)
-        elif scheduler_name == 'interval':
-            scheduler_config = ' '.join(scheduler_args)
-            return valid_interval_scheduler(scheduler_config, config_context)
-        elif scheduler_name == 'cron':
-            return valid_cron_scheduler(scheduler_args, config_context)
-        else:
-            return parse_groc_expression(schedule, config_context)
+    if 'type' not in schedule:
+        return schedule_config_from_legacy_dict(schedule, config_context)
 
-    if 'interval' in schedule:
-        return valid_interval_scheduler(schedule['interval'], config_context)
-    elif 'start_time' in schedule or 'days' in schedule:
-        start_time, days = schedule.get('start_time'), schedule.get('days')
-        return valid_daily_scheduler(start_time, days, config_context)
-    else:
-        path = config_context.path
-        raise ConfigError("Unknown scheduler at %s: %s" % (path, schedule))
+    schedule = ScheduleValidator().validate(schedule, config_context)
+    return validate_generic_schedule_config(schedule, config_context)
 
 
-def valid_daily_scheduler(time_string, days, config_context):
+def valid_constant_scheduler(_config, _context):
+    """Adapter for validation interface and constant scheduler."""
+    return ConfigConstantScheduler()
+
+
+def valid_daily_scheduler(config, config_context):
     """Daily scheduler, accepts a time of day and an optional list of days."""
+    schedule_config = config.value
+    time_string, days = pad_sequence(schedule_config.split(), 2)
     time_string = time_string or '00:00:00'
     time_spec   = config_utils.valid_time(time_string, config_context)
     days        = config_utils.valid_string(days or "", config_context)
@@ -76,47 +107,25 @@ def valid_daily_scheduler(time_string, days, config_context):
     original = "%s %s" % (time_string, days)
     weekdays = set(valid_day(day) for day in days or ())
     return ConfigDailyScheduler(original,
-        time_spec.hour, time_spec.minute, time_spec.second, weekdays)
+        time_spec.hour, time_spec.minute, time_spec.second, weekdays,
+        jitter=config.jitter)
 
 
 # Shortcut values for intervals
 TIME_INTERVAL_SHORTCUTS = {
-    'hourly': dict(hours=1),
+    'hourly': datetime.timedelta(hours=1)
 }
 
-# Translations from possible configuration units to the argument to
-# datetime.timedelta
-TIME_INTERVAL_UNITS = dicts.invert_dict_list({
-    'months':   ['mo', 'month', 'months'],
-    'days':     ['d', 'day', 'days'],
-    'hours':    ['h', 'hr', 'hrs', 'hour', 'hours'],
-    'minutes':  ['m', 'min', 'mins', 'minute', 'minutes'],
-    'seconds':  ['s', 'sec', 'secs', 'second', 'seconds']
-})
+def valid_interval_scheduler(config,  config_context):
+    def build_config(delta):
+        return ConfigIntervalScheduler(timedelta=delta, jitter=config.jitter)
 
-# Split digits and characters into tokens
-TIME_INTERVAL_RE = re.compile(r"^(?P<value>\d+)(?P<units>[a-zA-Z]+)$")
+    interval_key = config.value.strip()
+    if interval_key in TIME_INTERVAL_SHORTCUTS:
+        return build_config(TIME_INTERVAL_SHORTCUTS[interval_key])
 
-
-def valid_interval_scheduler(interval,  config_context):
-    interval    = ''.join(interval.split())
-    error_msg   = 'Invalid interval specification at %s: %s'
-
-    def build_config(spec):
-        return ConfigIntervalScheduler(timedelta=datetime.timedelta(**spec))
-
-    if interval in TIME_INTERVAL_SHORTCUTS:
-        return build_config(TIME_INTERVAL_SHORTCUTS[interval])
-
-    matches = TIME_INTERVAL_RE.match(interval)
-    if not matches:
-        raise ConfigError(error_msg % (config_context.path, interval))
-
-    units = matches.group('units')
-    if units not in TIME_INTERVAL_UNITS:
-        raise ConfigError(error_msg % (config_context.path, interval))
-
-    return build_config({TIME_INTERVAL_UNITS[units]: int(matches.group('value'))})
+    return build_config(
+        config_utils.valid_time_delta(config.value, config_context))
 
 
 def normalize_weekdays(seq):
@@ -219,11 +228,12 @@ DAILY_SCHEDULE_RE = build_groc_schedule_parser_re()
 def _parse_number(day):
     return int(''.join(c for c in day if c.isdigit()))
 
-def parse_groc_expression(expression, config_context):
+def parse_groc_expression(config, config_context):
     """Given an expression of the form in the docstring of
     daily_schedule_parser_re(), return the parsed values in a
     ConfigGrocScheduler
     """
+    expression = config.value
     m = DAILY_SCHEDULE_RE.match(expression.lower())
     if not m:
         msg = 'Schedule at %s is not a valid expression: %s'
@@ -260,14 +270,37 @@ def parse_groc_expression(expression, config_context):
         weekdays=weekdays,
         monthdays=monthdays,
         months=months,
-        timestr=timestr)
+        timestr=timestr,
+        jitter=config.jitter)
 
 
-def valid_cron_scheduler(scheduler_args, config_context):
+def valid_cron_scheduler(config, config_context):
     """Parse a cron schedule."""
     try:
-        crontab_kwargs = crontab.parse_crontab(' '.join(scheduler_args))
-        return ConfigCronScheduler(original=scheduler_args, **crontab_kwargs)
+        crontab_kwargs = crontab.parse_crontab(config.value)
+        return ConfigCronScheduler(
+            original=config.value, jitter=config.jitter, **crontab_kwargs)
     except ValueError, e:
         msg = "Invalid cron scheduler %s: %s"
         raise ConfigError(msg % (config_context.path, e))
+
+
+schedulers = {
+    'constant':     valid_constant_scheduler,
+    'daily':        valid_daily_scheduler,
+    'interval':     valid_interval_scheduler,
+    'cron':         valid_cron_scheduler,
+    'groc daily':   parse_groc_expression,
+}
+
+
+class ScheduleValidator(config_utils.Validator):
+    """Validate the structure of a scheduler config."""
+    config_class = ConfigGenericSchedule
+    defaults = {
+        'jitter':       datetime.timedelta()
+    }
+    validators = {
+        'type':         config_utils.build_enum_validator(schedulers.keys()),
+        'jitter':       config_utils.valid_time_delta
+    }
