@@ -55,6 +55,15 @@ class ActionGraph(object):
             return []
         return self.action_map[name].required_actions
 
+    def get_dependent_actions(self, name):
+        return self.action_map[name].dependent_actions
+
+    def get_actions(self):
+        return self.action_map.itervalues()
+
+    def get_action_map(self):
+        return self.action_map
+
     @property
     def names(self):
         return self.action_map.keys()
@@ -66,4 +75,4 @@ class ActionGraph(object):
         return self.graph == other.graph and self.action_map == other.action_map
 
     def __ne__(self, other):
-        return not self == other
\ No newline at end of file
+        return not self == other
