@@ -8,7 +8,7 @@ from tron import command_context
 from tron.core import action
 from tron.serialize import filehandler
 from tron import node
-from tron.actioncommand import ActionCommand
+from tron.actioncommand import ActionCommand, NoActionRunnerFactory
 
 from tron.utils import state, timeutils, proxy, iteration
 from tron.utils.observer import Observer
@@ -22,12 +22,12 @@ class ActionRunFactory(object):
     """
 
     @classmethod
-    def build_action_run_collection(cls, job_run):
+    def build_action_run_collection(cls, job_run, action_runner):
         """Create an ActionRunGraph from an ActionGraph and JobRun."""
+        action_map = job_run.action_graph.get_action_map().iteritems()
         action_run_map = dict(
-            (name, cls.build_run_for_action(job_run, action_inst))
-            for name, action_inst in job_run.action_graph.action_map.iteritems()
-        )
+            (name, cls.build_run_for_action(job_run, action_inst, action_runner))
+            for name, action_inst in action_map)
         return ActionRunCollection(job_run.action_graph, action_run_map)
 
     @classmethod
@@ -44,7 +44,7 @@ class ActionRunFactory(object):
         return ActionRunCollection(job_run.action_graph, action_run_map)
 
     @classmethod
-    def build_run_for_action(cls, job_run, action):
+    def build_run_for_action(cls, job_run, action, action_runner):
         """Create an ActionRun for a JobRun and Action."""
         run_node = action.node_pool.next() if action.node_pool else job_run.node
 
@@ -55,7 +55,8 @@ class ActionRunFactory(object):
             action.command,
             parent_context=job_run.context,
             output_path=job_run.output_path.clone(),
-            cleanup=action.is_cleanup)
+            cleanup=action.is_cleanup,
+            action_runner=action_runner)
 
     @classmethod
     def action_run_from_state(cls, job_run, state_data, cleanup=False):
@@ -123,18 +124,20 @@ class ActionRun(Observer):
 
     context_class               = command_context.ActionRunContext
 
+    # TODO: create a class for ActionRunId, JobRunId, Etc
     def __init__(self, job_run_id, name, node, bare_command=None,
             parent_context=None, output_path=None, cleanup=False,
             start_time=None, end_time=None, run_state=STATE_SCHEDULED,
-            rendered_command=None):
+            rendered_command=None, exit_status=None, action_runner=None):
         self.job_run_id         = job_run_id
         self.action_name        = name
         self.node               = node
         self.start_time         = start_time
         self.end_time           = end_time
-        self.exit_status        = None
+        self.exit_status        = exit_status
         self.bare_command       = bare_command
         self.rendered_command   = rendered_command
+        self.action_runner      = action_runner or NoActionRunnerFactory
         self.machine            = state.StateMachine(
                     self.STATE_SCHEDULED, delegate=self, force_state=run_state)
         self.is_cleanup         = cleanup
@@ -187,13 +190,14 @@ class ActionRun(Observer):
             start_time=state_data['start_time'],
             end_time=state_data['end_time'],
             run_state=state.named_event_by_name(
-                    cls.STATE_SCHEDULED, state_data['state'])
+                    cls.STATE_SCHEDULED, state_data['state']),
+            exit_status=state_data.get('exit_status')
         )
 
         # Transition running to fail unknown because exit status was missed
         if run.is_running:
             run._done('fail_unknown')
-        if run.is_queued or run.is_starting:
+        if run.is_starting:
             run.fail(None)
         return run
 
@@ -222,15 +226,23 @@ class ActionRun(Observer):
 
         return True
 
+    def stop(self):
+        stop_command = self.action_runner.build_stop_action_command(
+            self.id, 'terminate')
+        self.node.submit_command(stop_command)
+
+    def kill(self):
+        kill_command = self.action_runner.build_stop_action_command(
+            self.id, 'kill')
+        self.node.submit_command(kill_command)
+
     def build_action_command(self):
         """Create a new ActionCommand instance to send to the node."""
-        self.action_command = action_command = ActionCommand(
-            self.id,
-            self.command,
-            filehandler.OutputStreamSerializer(self.output_path)
-        )
-        self.watch(action_command)
-        return action_command
+        serializer = filehandler.OutputStreamSerializer(self.output_path)
+        self.action_command = self.action_runner.create(
+            self.id, self.command, serializer)
+        self.watch(self.action_command)
+        return self.action_command
 
     def handle_action_command_state_change(self, action_command, event):
         """Observe ActionCommand state changes."""
@@ -285,7 +297,8 @@ class ActionRun(Observer):
             'end_time':         self.end_time,
             'command':          command,
             'rendered_command': self.rendered_command,
-            'node_name':        self.node.name if self.node else None
+            'node_name':        self.node.get_name() if self.node else None,
+            'exit_status':      self.exit_status,
         }
 
     def render_command(self):
@@ -378,8 +391,8 @@ class ActionRunCollection(object):
                 proxy.func_proxy('fail',            iteration.list_all),
                 proxy.func_proxy('ready',           iteration.list_all),
                 proxy.func_proxy('cleanup',         iteration.list_all),
+                proxy.func_proxy('stop',            iteration.list_all),
                 proxy.attr_proxy('start_time',      iteration.min_filter),
-                proxy.attr_proxy('end_time',        iteration.max_filter),
             ])
 
     def action_runs_for_actions(self, actions):
@@ -476,6 +489,13 @@ class ActionRunCollection(object):
     def names(self):
         return self.run_map.keys()
 
+    @property
+    def end_time(self):
+        if not self.is_done:
+            return None
+        end_times = (run.end_time for run in self.get_action_runs_with_cleanup())
+        return iteration.max_filter(end_times)
+
     def __str__(self):
         def blocked_state(action_run):
             return ":blocked" if self._is_run_blocked(action_run) else ""
